#line 3679 "./lpsrc/flx_demux.pak"

#ifndef __DEMUX_QUITTER__
#define __DEMUX_QUITTER__

#include <flx_demux_config.hpp>
#include "demux_demuxer.hpp"  // demuxers
#include "pthread_mutex.hpp"  // mutexes
#include "pthread_condv.hpp"  // condition var for same

#ifdef _WIN32
#include "demux_wself_piper.hpp" // win32 self piper
#else
#include "demux_self_piper.hpp" // posix self piper
#endif

namespace flx { namespace demux {

// a waitable boolean. gratuitously tied to demuxer by demux_quit_flag
// todo: unhook.
class DEMUX_EXTERN async_bool : public demux_quit_flag {
  flx::pthread::flx_mutex_t cv_lock;       // to work with the condition var
  flx::pthread::flx_condv_t finished_cond;
  bool finished;   // might seem redundant, but that's how CVs work.
public:
  async_bool();

  void wait_until_true();
  virtual void signal_true();
};

// quits a demuxer
class DEMUX_EXTERN demux_quitter : public demux_callback {
  // self pipes for getting demuxer attention
#ifdef _WIN32
  wself_piper sp;
#else
  self_piper sp;
#endif
  async_bool finished;  // initially false
  void callback(demuxer* demux); // called back by demuxer in event thread.
public:
  void quit(demuxer* demux); // blocks until event thread exits
};

/*
class DEMUX_EXTERN wdemux_quitter : demux_callback {
  wself_piper sp;
  async_bool finished; // initially false
public:
  void quit(iocp_demuxer* demux); // blocks until demuxer/event thread exits

  void callback(demuxer* d); // called back by demuxer
};
*/

} }

#endif

